from base64 import urlsafe_b64decode, urlsafe_b64encode
from html import escape, unescape
from . import utils

import hashlib
import urllib.parse


def caeser(data, shift: int = 5, decode: bool = False) -> str:
    """
    Caeser's Cipher used on a string.
    """

    if decode:
        shift = -shift

    result = ""

    for char in data:
        if utils.is_not_ascii_letter(character=char):  # Account for spaces and special chars.
            result += char
        elif char.isupper():
            enciphered_index = (utils.ascii_to_index(letter=char) + shift) % 26
            result += utils.index_to_ascii(index=enciphered_index, capitalize=True)
        else:
            enciphered_index = (utils.ascii_to_index(letter=char) + shift) % 26
            result += utils.index_to_ascii(index=enciphered_index)
    return result


def base64(data: str, decode: bool = False) -> str:
    """
    By default, encode into Base64 string.
    """

    if decode:
        return urlsafe_b64decode(data).decode()

    encoded = urlsafe_b64encode(data.encode('utf-8'))
    return encoded.decode()


def full_url(data: str, decode: bool = False) -> str:

    if decode:
        url_decoded = data.replace('%2E', '.')
        return urllib.parse.unquote(url_decoded)

    url_encode = urllib.parse.quote(data)
    return url_encode.replace('.', '%2E')


def basic_hex(data, decode: bool = False) -> str:
    """
    By default, encode into Hexadecimal string.
    """
    if decode:
        return bytes.fromhex(data).decode()

    bytes_conversion = data.encode()
    return bytes_conversion.hex()


def html(data, decode: bool = False) -> str:
    """
    By default, encode into HTML safe string.
    """
    if decode:
        return unescape(data)

    return escape(data)


def md5(data):
    """
    Encrypt string into MD5 hash.
    """
    hashed_value = hashlib.md5(data.encode())
    return hashed_value.hexdigest()






from select import select
from threading import Thread
from time import sleep

import socket

HEADER = "[SC] "

CONNECTION_CLOSED = f"{HEADER}Connection closed."
NO_CLIENT = f"{HEADER}No client connections."
NO_DATA = f"{HEADER}No data was provided to send."
SENT = f"{HEADER}Sent."
SERVER_STARTED = f"{HEADER}Server started."
TIMEOUT_RECEIVED = f"{HEADER}Send / Receive timed out."


class QuickSocketListener:
    """
    Listener designed to work with Netcat reverse shells.
    """
    def __init__(
            self,
            local_host: str = '0.0.0.0',
            local_port: int = 5000,
            timeout_in_sec: int = 5,
            carriage_return: str = '\n',
            print_message: bool = True
    ):

        self.local_host = local_host
        self.local_port = local_port
        self.timeout_in_sec = timeout_in_sec
        self.carriage_return = carriage_return
        self.print_message = print_message

        self.buffer_size = 1024 * 25
        self.max_clients = 5

        self.server: socket = None
        self.clients: list[socket] = []

        self.active_session: socket = None

    def _bind_socket(self) -> None:
        """
        Start listening socket server.
        """
        try:

            self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.server.bind((self.local_host, self.local_port))
            self.server.listen(self.max_clients)

            print(f"Listening for connection(s)...")

            while True:
                client_socket, client_address = self.server.accept()
                self.clients.append(client_socket)

                if not self.active_session:
                    self.active_session = client_socket

                print(f"Connection: {client_address[0]}:{client_address[1]}")

        except (socket.error, KeyboardInterrupt):
            return f'[-] Problem encountered: {socket.error} '

    def _display_message(self, message: str):
        if self.print_message:
            print(message)
        return message

    def close(self, client_session: socket = None, timer: int = 1):
        """
        Exit client connection and close connection.
        Also, clean up 'self.clients' list and reset 'self.active_session'.
        """

        if timer:
            sleep(timer)

        if not client_session:
            client_session = self.active_session

        try:
            self.clients.remove(client_session)
            self.active_session = None
            client_session.shutdown(socket.SHUT_RDWR)
            client_session.close()

            return self._display_message(message=CONNECTION_CLOSED)
        except (socket.error, OSError, ValueError):
            pass

    def send(self, data: str, client_session: socket = None) -> str:
        """
        Send 'data' to the client session.
        """

        if not self.clients:
            return self._display_message(message=NO_CLIENT)

        if not data:
            return self._display_message(message=NO_DATA)

        if not client_session:
            client_session = self.active_session

        client_session.send(f"{data}{self.carriage_return}".encode())

        return self._display_message(message=SENT)

    def send_recv(self, data: str, client_session: socket = None) -> str:
        """
        Typically used as a Listener for shellcode connections.

        Send 'data' to the client session.
        Return received data.
        """

        if not self.clients:
            return self._display_message(message=NO_CLIENT)

        if not data:
            return self._display_message(message=NO_DATA)

        if not client_session:
            client_session = self.active_session

        client_session.send(f"{data}{self.carriage_return}".encode())

        # Confirm data is being sent with a timeout
        ready = select([client_session], [], [], self.timeout_in_sec)

        if ready[0]:
            message = client_session.recv(self.buffer_size).decode()
            return self._display_message(message=message)

        return self._display_message(message=TIMEOUT_RECEIVED)

    def start(self, timer: int = 1):
        """
        Startup SummoningCircle in the background.
        """
        listener = Thread(target=self._bind_socket)
        listener.start()
        sleep(timer)

        return self._display_message(message=SERVER_STARTED)

    def stop(self):
        """
        Close client connections and shutdown server.
        """
        try:
            for client in self.clients:
                client.close()
            self.server.shutdown(socket.SHUT_RDWR)
        except (socket.error, OSError, ValueError):
            pass
        self.server.close()

    def receive(self, client_session: socket = None) -> str:
        """
        Default way to receive data.
        """

        if not client_session:
            client_session = self.active_session

        # Confirm data is being sent with a timeout
        ready = select([client_session], [], [], self.timeout_in_sec)

        if ready[0]:
            message = client_session.recv(self.buffer_size).decode()
            return self._display_message(message=message)

        return self._display_message(message=TIMEOUT_RECEIVED)

    def recv_send(self, data: str, client_session: socket = None) -> str:
        """
        Receive data.
        Send 'data' to the client session.
        Return received data.
        """

        if not self.clients:
            return self._display_message(message=NO_CLIENT)

        if not data:
            return self._display_message(message=NO_DATA)

        if not client_session:
            client_session = self.active_session

        # Confirm data is being sent with a timeout
        ready = select([client_session], [], [], self.timeout_in_sec)

        if ready[0]:
            client_session.send(f"{data}{self.carriage_return}".encode())
            message = client_session.recv(self.buffer_size).decode()
            return self._display_message(message=message)

        return self._display_message(message=TIMEOUT_RECEIVED)



import socket
import platform
import netifaces


def get_private_ips():
    interfaces = netifaces.interfaces()

    local_ipv4 = dict()
    for interface in interfaces:

        ipv4_info = netifaces.ifaddresses(interface).get(2)
        ipv4_addr = ipv4_info[0].get('addr')
        local_ipv4[interface] = ipv4_addr

    return local_ipv4


def os_specs():
    hostname = socket.gethostname()

    return {
        "hostname": hostname,
        "hostByHostname": socket.gethostbyname(hostname),
        "hostByAddress": list(socket.gethostbyname_ex(hostname)),
        "localIpV4": get_private_ips(),
        "platform": {
            "system": platform.system(),
            "version": platform.version(),
            "platform": platform.platform(),
            "machine": platform.machine(),
            "processor": platform.processor()
        }
    }



from concurrent.futures import ThreadPoolExecutor
from http.server import HTTPServer, SimpleHTTPRequestHandler
import time


class QuickHttpServer:
    def __init__(self, ip: str = '', port: int = 8000):
        """
        The HTTP server starts when this class is instantiated.
        This allows users the ability to start an HTTP server for file transfer in the background.
        """
        server_address = (ip, port)

        server_class = HTTPServer
        self.httpd = server_class(server_address, SimpleHTTPRequestHandler)

        executor = ThreadPoolExecutor(max_workers=1)
        executor.submit(self.httpd.serve_forever)

    def stop(self):
        self.httpd.shutdown()


if __name__ == '__main__':
    hs = QuickHttpServer()
    time.sleep(5)
    hs.stop()



from string import ascii_letters


def ascii_to_index(letter: str) -> int:
    return ord(letter.lower()) - 97


def index_to_ascii(index: int, capitalize: bool = False) -> str:
    index = index % 26
    if capitalize:
        return chr(index + 65)
    return chr(index + 97)


def is_ascii_letter(character: str) -> bool:
    return character in ascii_letters


def is_not_ascii_letter(character: str) -> bool:
    return character not in ascii_letters



from setuptools import setup

# python setup.py sdist
# twine upload dist/*

setup(name='CyberBook',
      version='0.0.26',
      description='Cyber spellbook for developers.',
      keywords='cyberbook DecoderRing tools',
      author='Clutch_Reboot',
      author_email='clutchshadow26@gmail.com',
      license='GNU General Public License v3.0',
      packages=[
            'CyberBook',
            'CyberBook.Background',
            'CyberBook.Lists',
            'CyberBook.DecoderRing',
            'CyberBook.Identify',
            'CyberBook.Misc'
      ],
      zip_safe=False,
      long_description=open('README.md', 'rt').read(),
      long_description_content_type='text/markdown',
      python_requires='>=3.10',
      install_requires=[
            'requests',
            'urllib3',
            'netifaces'
      ],
      project_urls={
            "Documentation": "https://clutchreboot.github.io/CyberBookDocs/",
            "Source": "https://github.com/ClutchReboot/CyberBook",
      },
      )
